-- ============================================================================
-- == HTBrokerSupport - Provides support for integrating this addon into     ==
-- ==                   LibDataBroker (aka: LDB) supporting display addons.  ==
-- ============================================================================

HTBrokerSupport    = LibStub("AceAddon-3.0"):NewAddon("HTBrokerSupport", "AceConsole-3.0", "AceEvent-3.0")
local L            = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local DropDownMenu = CreateFrame("Frame", "DropDownMenu")       -- Dropdown Menu
local info         = {}                                 -- Dropdown Menu Options

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTBrokerSupport:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTBrokerSupport:OnEnable()

  HTBrokerSupport:CreateDataObject()

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTBrokerSupport:OnDisable()
end

-- ============================================================================
-- == CreateDataObject()                                                     ==
-- ==                                                                        ==
-- == This function creates the LDB data object.                             ==
-- ============================================================================

function HTBrokerSupport:CreateDataObject()

  LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("HalonaTools", {
    type = "launcher",
    label = L["addon-name"],
    OnClick = function(_, msg)
      if msg == "LeftButton" then
        HalonaTools:JoinGuildRaidEvent()
      elseif msg == "RightButton" then
        HTBrokerSupport:ShowMenu()
      end
    end,
    icon = "Interface\\AddOns\\HalonaTools\\Images\\icon",
    OnTooltipShow = function(tooltip)
      if not tooltip or not tooltip.AddLine then return end
      tooltip:AddLine("|r" .. L["addon-name"] .. " (" .. L["addon-version-number"] .. ")|r")
      tooltip:AddLine(" ")
      tooltip:AddLine(L["addon-desc-01"])
      tooltip:AddLine(L["addon-desc-02"])
      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-guildinfo"] .. "|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-name"]  .. "|r", "|cff00ff00" .. HTCommon:GetGuildName()     .."|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-rank"]  .. "|r", "|cff00ff00" .. HTCommon:GetGuildRankName() .."|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-level"] .. "|r", "|cff00ff00" .. HTCommon:GetGuildLevel()    .."|r")
      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-guildofficers"] .. "|r")

      local guildOfficerList = HTCommon:GetGuildOfficerList()
      local colouration      = "|cff00ff00"

      for i,v in ipairs(guildOfficerList) do

        -- TODO: Replace guildOfficerList[i] with v.
        local officerName = HTCommon:FindPlayerByCharacterName(guildOfficerList[i])
        if not strfind (officerName, L["con-findbycharname-offline"]) == nil then
          colouration = "|cffff0000"
        end

        tooltip:AddLine(colouration .. officerName .. "|r")

      end
      
      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-tools"] .. "|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-autoinvitations"]  .. "|r", "|cff00ff00" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoInvitations(nil)) .. "|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-rms-timer"]  .. "|r", "|cffffff00" .. " (".. HTData:SecondsToMinutes(HTConsole:GetRMSTimer()) .. ") " .. "|r" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRecruitmentMessageSender(nil)))

      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-actions"] .. "|r")
      tooltip:AddDoubleLine("|cffeda55f" .. L["ldb-tooltip-item-name-leftclick"]  .. "|r", "|cff00ff00" .. L["ldb-tooltip-item-value-leftclick"] .. "|r")
      tooltip:AddDoubleLine("|cffeda55f" .. L["ldb-tooltip-item-name-rightclick"] .. "|r", "|cff00ff00" .. L["ldb-tooltip-item-value-rightclick"]  .. "|r")
    end,
  })

end

-- ============================================================================
-- == ShowMenu()                                                             ==
-- ==                                                                        ==
-- == This function displays the drop-down menu.                             ==
-- ============================================================================

function HTBrokerSupport:ShowMenu()

  HTBrokerSupport:BuildDropDownMenu()

  ToggleDropDownMenu(1, nil, DropDownMenu, "cursor", 0, 0)

end

-- ============================================================================
-- == BuildDropDownMenu()                                                    ==
-- ==                                                                        ==
-- == This function builds the drop-down menu. It is a dynamic function and  ==
-- == can be called either once only for menus that are not expected to      ==
-- == change during a session or every time before the menu is displayed for ==
-- == menus that change often.                                               ==
-- ============================================================================

function HTBrokerSupport:BuildDropDownMenu()

  DropDownMenu.displayMode = "MENU"
  DropDownMenu.initialize  = function(self, level)

  if not level then return end
  wipe(info)

  -- ==========================================================================
  -- == Level 1 - Top level menu items.                                      ==
  -- ==========================================================================
  if level == 1 then

    -- General Menu Defaults.
    HTBrokerSupport:AddMenuDefaultConfig()

    -- Create the title of the menu
    HTBrokerSupport:AddMenuTitle(L["ldb-menu-main-name"])
    UIDropDownMenu_AddButton(info, level)

    -- Empty Menu Item.
    HTBrokerSupport:AddMenuDummyButton()
    UIDropDownMenu_AddButton(info, level)

    -- Buff-Monitor Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuBuffMonitor")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-buffmonitor")
    UIDropDownMenu_AddButton(info, level)

    -- Invitations Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuInvitations")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-invitations")
    UIDropDownMenu_AddButton(info, level)

    -- Raid Functions Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuRaidFunctions")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-raid")
    UIDropDownMenu_AddButton(info, level)

    -- Recruitment Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuRecruitment")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-recruitment")
    UIDropDownMenu_AddButton(info, level)

    -- System Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuSystem")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-system")
    UIDropDownMenu_AddButton(info, level)

    -- Empty Menu Item.
    HTBrokerSupport:AddMenuDummyButton()
    UIDropDownMenu_AddButton(info, level)

    -- About Menu Item.
    HTBrokerSupport:AddMenuPushButton()
    HTBrokerSupport:AddMenuButtonText("ldb-menu-item-about")
    info.func = function() HTGUI:About() end
    UIDropDownMenu_AddButton(info, level)

    -- Close Menu Item.
    HTBrokerSupport:AddMenuPushButton()
    HTBrokerSupport:AddMenuButtonText("ldb-menu-item-closemenu")
    info.func = function() CloseDropDownMenus() end
    UIDropDownMenu_AddButton(info, level)

  -- ==========================================================================
  -- == Level 2 - Submenus directly linked from the main menu.               ==
  -- ==========================================================================
  elseif level == 2 then

    -- ========================================================================
    -- == Render the level 2 submenu for: Buff-Monitor ========================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBuffMonitor" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-buffmonitor-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Do Self Check.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doselfbuffcheck")
      info.func = function() HTBuffMonitor:DoSelfCheck() end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Do Flask Check Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doflaskcheck")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitorCheckFlasks()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-doflaskcheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do Battle Elixir Check Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dobelixircheck")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitorCheckBattleElixirs()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-dobelixircheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

       -- Do Guardian Elixir Check Toggle.
       HTBrokerSupport:AddMenuToggleButton()
       HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dogelixircheck")
       info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil))
       info.func    = function()
                        HTConfig:ToggleIsEnabledBuffMonitorCheckGuardianElixirs()
                        HTBrokerSupport:DisplayStatus(L["ldb-menu-item-dogelixircheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil)))
                      end
      UIDropDownMenu_AddButton(info, level)

    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Raid Functions ======================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRaidFunctions" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-raid-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Begin Invitation Period Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-begin-invites")
      info.func = function() HalonaTools:BeginRaidInvitationPeriod(UnitName("player")) end
      UIDropDownMenu_AddButton(info, level)

      -- Begin Raid Session Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-begin-raid")
      info.func = function() HalonaTools:BeginRaidSession(UnitName("player")) end
      UIDropDownMenu_AddButton(info, level)

      -- End Raid Session Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-end-raid")
      info.func = function() HalonaTools:EndRaidSession(UnitName("player")) end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 5 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-05m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(300, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "5"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 10 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-10m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(600, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "10"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 15 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-15m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(900, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "15"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 20 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-20m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(1200, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "20"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 25 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-25m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(1500, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "25"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 30 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-30m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(1800, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "30"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Invitations =========================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuInvitations" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-invitations-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Auto Invitations Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoinvitations")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoInvitations(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoInvitations()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoinvitations-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoInvitations(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Auto Raid Promotions Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoraidpromotions")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoRaidPromotions(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoRaidPromotions()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoraidpromotions-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRaidPromotions(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Auto Raid Invitation Announcement Response.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoriaresponse")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoRIAResponse(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoRIAResponse()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoriaresponse-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRIAResponse(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Auto Raid Invitation Announcement Response.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autorqaresponse")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoRQAResponse(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoRQAResponse()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autorqaresponse-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRQAResponse(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)
    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Recruitment =========================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRecruitment" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-recruitment-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Set NGR Message Button.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-setngrmessage")
      info.func = function()
                    HTConsole:ExecuteChatCommand("/ngr msg " .. L["ldb-menu-item-setngrmessage-msg"])
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Send RMS Now Button.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-manualrms")
      info.func = function()
                    HTConsole:BroadcastMessageRMS()
                  end
      UIDropDownMenu_AddButton(info, level)
      
      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Recruitment Message Sender Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autorms")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledRecruitmentMessageSender(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledRecruitmentMessageSender()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autorms-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRecruitmentMessageSender(nil)))
                       HTConsole:ToggleRMSTimerHelper()
                     end
      UIDropDownMenu_AddButton(info, level)
      
    end

    -- ========================================================================
    -- == Render the level 2 submenu for: System ==============================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuSystem" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-system-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Show Report Addon Usage (CHAT).
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-chat")
      info.func = function()
                    HalonaTools:ShowReportUsageStatus("CHAT")
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Show Report Addon Usage (GUILD).
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-guild")
      info.func = function()
                    HalonaTools:ShowReportUsageStatus("GUILD")
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Show Report Addon Usage (RAID).
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-raid")
      info.func = function()
                    HalonaTools:ShowReportUsageStatus("RAID")
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Addon Version Broadcast.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-avb")
      info.func = function()
                    HTConsole:BroadcastMessageAVB()
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Addon Version Query.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-avq")
      info.func = function()
                    HTConsole:BroadcastMessageAVQ(L["ldb-menu-item-avq-messagetext"])
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Addon Version Query.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-sim-newer-addon")
      info.func = function()
                    HTConsole:OldVersionCheck("Whizz-Bang-Wallop", "10.0")
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Alert New Addon Version Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-alertnewaddonversion")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAlertNewAddonVersion(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAlertNewAddonVersion()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-alertnewaddonversion-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAlertNewAddonVersion(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Debug Mode Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugmode")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledDebugMode(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledDebugMode()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-debugmode-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugMode(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Config Menu Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-config")
      info.func = function()
                    HTGUI:Config()
                  end
      UIDropDownMenu_AddButton(info, level)
    end
    end
  end
end

-- ============================================================================
-- == AddMenuDefaultConfig()                                                 ==
-- ==                                                                        ==
-- == This function adds the default meta-data required by all menu items.   ==
-- ============================================================================

function HTBrokerSupport:AddMenuDefaultConfig()

  info.keepShownOnClick = 1

end

-- ============================================================================
-- == AddMenuTitle(title)                                                    ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new menu title.        ==
-- ============================================================================

function HTBrokerSupport:AddMenuTitle(title)

  -- Create the title of the menu
  info.isTitle      = 1
  info.text         = title
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuDummyButton()                                                   ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new dummy button.      ==
-- ============================================================================

function HTBrokerSupport:AddMenuDummyButton()

  -- Empty Menu Item.
  info.text         = " "
  info.checked      = nil
  info.notCheckable = 1
  info.hasArrow     = nil
  info.value        = nil
  info.disabled     = 1

end

-- ============================================================================
-- == AddMenuSubMenu()                                                       ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new sub-menu.          ==
-- ============================================================================

function HTBrokerSupport:AddMenuSubMenu(name)

  info.isTitle  = nil
  info.disabled = nil
  info.func     = self.UncheckHack
  info.hasArrow = 1
  info.value    = name

end

-- ============================================================================
-- == AddMenuPushButton()                                                    ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new push button.       ==
-- ============================================================================

function HTBrokerSupport:AddMenuPushButton()

  info.isTitle      = nil
  info.disabled     = nil
  info.hasArrow     = nil
  info.value        = nil
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuToggleButton()                                                  ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new toggle button.     ==
-- ============================================================================

function HTBrokerSupport:AddMenuToggleButton()

  info.isTitle      = nil
  info.disabled     = nil
  info.hasArrow     = nil
  info.notCheckable = nil

end

-- ============================================================================
-- == AddMenuButtonText(referenceStub)                                       ==
-- ==                                                                        ==
-- == This function adds menu button text fields to the 'info' table for the ==
-- == supplied referenceStub value.                                          ==
-- ============================================================================

function HTBrokerSupport:AddMenuButtonText(localeString)

  local txtName = localeString .. "-txt-name"
  local tipName = localeString .. "-tip-name"
  local tipDesc = localeString .. "-tip-desc"

  info.text         = L[txtName]
  info.tooltipTitle = L[tipName]
  info.tooltipText  = L[tipDesc]

end

-- ============================================================================
-- == IsEnabled(value)                                                       ==
-- ==                                                                        ==
-- == This function determines whether the supplied value is recognised as   ==
-- == meaning "is enabled" and returns '1' if enabled, nil otherwise.        ==
-- ============================================================================

function HTBrokerSupport:IsEnabled(value)

  if value == true or value == 1 then
    return 1
  else
    return nil
  end

end

-- ============================================================================
-- == IsEnabledText(value)                                                   ==
-- ==                                                                        ==
-- == This function determines whether the supplied value is recognised as   ==
-- == meaning "is enabled" and returns 'On' if enabled, 'Off' otherwise.     ==
-- ============================================================================

function HTBrokerSupport:IsEnabledText(value)

  if HTBrokerSupport:IsEnabled(value) then
    return "|cff00ff00On|r"
  else
    return "|cffff0000Off|r"
  end

end

-- ============================================================================
-- == DisplayStatus(featureName, isEnabledText)                              ==
-- ==                                                                        ==
-- == This function enables the status of a feature to be displayed via the  ==
-- == standard chat-window system with appropriate colourisation.            ==
-- ============================================================================

function HTBrokerSupport:DisplayStatus(featureName, isEnabledText)

  local message = "[HT] - " .. featureName .. ": " .. isEnabledText

  HTConsole:SendChatMessage(message, "CHAT", nil, nil)

end
