-- ============================================================================
-- == HTGUI - Provides support to HalonaTools for managing GUI resources.    ==
-- ============================================================================

HTGUI        = LibStub("AceAddon-3.0"):NewAddon("HTGUI", "AceConsole-3.0", "AceEvent-3.0")
local L      = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local AceGUI = LibStub("AceGUI-3.0")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTGUI:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTGUI:OnEnable()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTGUI:OnDisable()
end

-- ============================================================================
-- == AboutHalonaTools()                                                     ==
-- ==                                                                        ==
-- == This function displays a standard "About" window, providing            ==
-- == information about the HalonaTools addon.                               ==
-- ============================================================================

function HTGUI:About()

  local frame = AceGUI:Create("Frame")
  frame:SetTitle("About")
  frame:SetStatusText("(c) aLL I EVER WANTED productions, 2011.")

end

-- ============================================================================
-- == Config()                                                               ==
-- ==                                                                        ==
-- == This function displays the configuration window for the HalonaTools    ==
-- == addon.                                                                 ==
-- ============================================================================

function HTGUI:Config()

  -- function that draws the widgets for the first tab
  local function DrawGroup1(container)
    local desc = AceGUI:Create("Label")
    desc:SetText("This is Tab 1")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local button = AceGUI:Create("Button")
    button:SetText("Tab 1 Button")
    button:SetWidth(200)
    container:AddChild(button)
  end

  -- function that draws the widgets for the second tab
  local function DrawGroup2(container)
    local desc = AceGUI:Create("Label")
    desc:SetText("This is Tab 2")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local button = AceGUI:Create("Button")
    button:SetText("Tab 2 Button")
    button:SetWidth(200)
    container:AddChild(button)
  end

  -- Callback function for OnGroupSelected
  local function SelectGroup(container, event, group)
     container:ReleaseChildren()
     if group == "tab1" then
        DrawGroup1(container)
     elseif group == "tab2" then
        DrawGroup2(container)
     end
  end

  -- Create the frame container
  local frame = AceGUI:Create("Frame")
  frame:SetTitle("Example Frame")
  frame:SetStatusText("AceGUI-3.0 Example Container Frame")
  frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
  -- Fill Layout - the TabGroup widget will fill the whole frame
  frame:SetLayout("Fill")

  -- Create the TabGroup
  local tab =  AceGUI:Create("TabGroup")
  tab:SetLayout("Flow")
  -- Setup which tabs to show
  tab:SetTabs({{text="Tab 1", value="tab1"}, {text="Tab 2", value="tab2"}})
  -- Register callback
  tab:SetCallback("OnGroupSelected", SelectGroup)
  -- Set initial Tab (this will fire the OnGroupSelected callback)
  tab:SelectTab("tab1")

  -- add to the frame container
  frame:AddChild(tab)

end


-- ============================================================================
-- == PopupAskRequestRaidInvitation(raidInviter, messageAnnouncer)           ==
-- ==                                                                        ==
-- == This function displays a static popup dialog asking the player if they ==
-- == wish to request a raid invitation from the current raid inviter and    ==
-- == processes the response they make.                                      ==
-- ============================================================================

function HTGUI:PopupAskRequestRaidInvitation(raidInviter, messageAnnouncer)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the text strings...                                    --
  -- == ----------------------------------------------------------------------
  local windowTitle   = string.gsub(L["gui-ria-window-msg-01"], "RIA_INVITER", raidInviter)
  local joinMessage   = L["gui-ria-msg-first"] .. L["gui-ria-join-msg-01"]
  local cancelMessage = L["gui-ria-msg-first"] .. L["gui-ria-cancel-msg-01"]

  cancelMessage = string.gsub(cancelMessage, "RIA_KEYWORD", L["gui-ria-keyword"])
  cancelMessage = string.gsub(cancelMessage, "RIA_INVITER", raidInviter)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the popup window...                                    --
  -- == ----------------------------------------------------------------------
  StaticPopupDialogs[L["gui-ria-window-title"]] = {

    text         = windowTitle,
    button1      = L["gui-ria-window-btn-yes"],
    button2      = L["gui-ria-window-btn-no"],
    timeout      = 30,
    whileDead    = 1,
    hideOnEscape = 0,

    OnAccept = function()
      HTConsole:SendChatMessage(L["gui-ria-keyword"], "WHISPER", nil, raidInviter)
      HTConsole:SendChatMessage(joinMessage, "CHAT", nil, nil)
    end,

    OnCancel = function()
      HTConsole:SendChatMessage(cancelMessage, "CHAT", nil, nil)
    end
  }

  -- == ----------------------------------------------------------------------
  -- == -- Display the popup window...                                      --
  -- == ----------------------------------------------------------------------
  if HTConfig:GetIsEnabledAutoRIAResponse(nil) then
    StaticPopup_Show (L["gui-ria-window-title"])
    PlaySound("DwarfExploration")
  end

end

-- ============================================================================
-- == PopupAskRequestRaidQueueListing(raidInviter, messageAnnouncer)         ==
-- ==                                                                        ==
-- == This function displays a static popup dialog asking the player if they ==
-- == wish to request a raid queue listing from the current raid queue       ==
-- == manager and processes the response they make.                          ==
-- ============================================================================

function HTGUI:PopupAskRequestRaidQueueListing(raidInviter, messageAnnouncer)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the text strings...                                    --
  -- == ----------------------------------------------------------------------
  local windowTitle   = string.gsub(L["gui-rqa-window-msg-01"], "RQA_INVITER", raidInviter)
  local joinMessage   = L["gui-rqa-msg-first"] .. L["gui-rqa-join-msg-01"]
  local cancelMessage = L["gui-rqa-msg-first"] .. L["gui-rqa-cancel-msg-01"]

  cancelMessage = string.gsub(cancelMessage, "RQA_KEYWORD", L["gui-rqa-keyword"])
  cancelMessage = string.gsub(cancelMessage, "RQA_INVITER", raidInviter)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the popup window...                                    --
  -- == ----------------------------------------------------------------------
  StaticPopupDialogs[L["gui-rqa-window-title"]] = {

    text         = windowTitle,
    button1      = L["gui-rqa-window-btn-yes"],
    button2      = L["gui-rqa-window-btn-no"],
    timeout      = 30,
    whileDead    = 1,
    hideOnEscape = 0,

    OnAccept = function()
      HTConsole:SendChatMessage(L["gui-rqa-keyword"], "WHISPER", nil, raidInviter)
      HTConsole:SendChatMessage(joinMessage, "CHAT", nil, nil)
    end,

    OnCancel = function()
      HTConsole:SendChatMessage(cancelMessage, "CHAT", nil, nil)
    end
  }

  -- == ----------------------------------------------------------------------
  -- == -- Display the popup window...                                      --
  -- == ----------------------------------------------------------------------
  if HTConfig:GetIsEnabledAutoRIAResponse(nil) then
    StaticPopup_Show (L["gui-rqa-window-title"])
    PlaySound("DwarfExploration")
  end

end
